/*
 *  File: JaretDate.java 
 *  Copyright (c) 2004-2005  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.util.date;


import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import sun.util.calendar.CalendarDate;

import de.jaret.util.misc.PropertyObservableBase;




/**
 * A wrapper for <code>java.util.Date</code> and <code>java.util.Calendar</code> to ease the use
 * of date and time. There are some convenience methods for quick textual output. They are located in a delegate
 * that supports i8n. Use <code>setLocale</code> to set the locale of the delegate. The delegate is static and any
 * change will affect the output of all JaretDates!
 * @author Peter Kliem
 * @version $Id: JaretDate.java 125 2005-06-28 10:40:03Z kliem $
 */
public class JaretDate extends PropertyObservableBase{
	public static final long MILLIS_PER_HOUR = 60 * 60 * 1000;
	public static final long MILLIS_PER_MINUTE = 60 * 1000;
	public static final long MILLIS_PER_SECOND = 1000;
	
	/**
	 * Delegate for date formatting and parsing
	 */
	private static JaretDateFormatter _jaretDateFormatter = new JaretDateFormatter();
	
	// encapsulated Date
	private Date _date;
	// Calendar will only be instantiated if necessary
	private Calendar _calendar;
	
	/**
	 * Constructs a JaretDate with the current date and time
	 */
	public JaretDate() {
		_date = new Date();
	}
	/**
	 * Constructs a JaretDate using a clone of the given Date
	 * @param date initial date to set
	 */
	public JaretDate(Date date) {
		_date = (Date)date.clone();
	}

	/**
	 * Constructs a JaretDate by milliseconds given in a String
	 * @param millisecondsString as String
	 */
	public JaretDate(String millisecondsString) {
		long ms = Long.parseLong(millisecondsString);
		_date = new Date(ms);
	}
	/**
	 * Constructs a JaretDate using the given millesecond value
	 * @param milliseconds as long
	 */
	public JaretDate(long milliseconds) {
		long ms = milliseconds;
		_date = new Date(ms);
	}
	
	/**
	 * Constructs a new JaretDate object with the same time as the given jaretDate
	 * @param jdate JaretDate to copy
	 */
	public JaretDate(JaretDate jdate) {
		_date = new Date(jdate.getDate().getTime());
	}
	/**
	 * Constructs a JaretDate setting it to a given date and time
	 * @param day day of month
	 * @param month month of year
	 * @param year year
	 * @param hours hours in day
	 * @param minutes minutes in hour
	 * @param seconds seconds in minute
	 */
	public JaretDate(int day, int month, int year, int hours, int minutes, int seconds) {
		this();
		setDateTime(day, month, year, hours, minutes, seconds);
	}
	
	/**
	 * Retrieve the configured JaretDateFormatter.
	 * @return JaretDateFormatter used for formatting and parsing
	 */
	public JaretDateFormatter getJaretDateFormatter() {
		return _jaretDateFormatter;
	}
	
	/**
	 * Sets the JaretDateFormatter to be used for parsing and formatting. The JaretDateFormatter is used for all
	 * JaretDates! A Change will affect parsing and formatting of all JaretDates! 
	 * @param jaretDateFormatter the new Formatter to be used.
	 */
	public static void setJaretDateFormatter(JaretDateFormatter jaretDateFormatter) {
//		JaretDateFormatter oldVal = _jaretDateFormatter;
		_jaretDateFormatter = jaretDateFormatter;
		// if we detect a real change, fire a prop change thus enabling listeners to redraw
		// textual representations
/*		if (!oldVal.equals(jaretDateFormatter)) {
			firePropertyChange();
		}
*/	}
	/**
	 * Set the locale of the JaretDateFormatter. Calling will fire a property change for the date.
	 * @param locale Locale to be used
	 */
	public void setLocale(Locale locale) {
		_jaretDateFormatter.setLocale(locale);
		firePropertyChange();
	}
	
	/**
	 * Sets the date and time of the JaretDate by values
	 * @param day day of month
	 * @param month month of year
	 * @param year year
	 * @param hours hours in day
	 * @param minutes minutes in hour
	 * @param seconds seconds in minute
	 */
	public void setDateTime(int day, int month, int year, int hours, int minutes, int seconds) {
		Calendar cal = getCalendar();
		cal.set(Calendar.DAY_OF_MONTH, day);
		cal.set(Calendar.MONTH, month-1);
		cal.set(Calendar.YEAR, year);
		cal.set(Calendar.HOUR_OF_DAY, hours);
		cal.set(Calendar.MINUTE, minutes);
		cal.set(Calendar.SECOND, seconds);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
	}

	/**
	 * Set the time. Does not affect the date. Will set the milliseconds to zero. 
	 * @param hours
	 * @param minutes
	 * @param seconds
	 */
	public void setTime(int hours, int minutes, int seconds) {
		Calendar cal = getCalendar();
		cal.set(Calendar.HOUR_OF_DAY, hours);
		cal.set(Calendar.MINUTE, minutes);
		cal.set(Calendar.SECOND, seconds);
		cal.set(Calendar.MILLISECOND, 0);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
	}
	 /**
	  * Set the date. Does not affect the time.
	  * @param day
	  * @param month
	  * @param year
	  */
	public void setDate(int day, int month, int year) {
		Calendar cal = getCalendar();
		cal.set(Calendar.DAY_OF_MONTH, day);
		cal.set(Calendar.MONTH, month-1);
		cal.set(Calendar.YEAR, year);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
	}

	public void setDay(int day) {
		Calendar cal = getCalendar();
		cal.set(Calendar.DAY_OF_MONTH, day);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
	}

	/**
	 * Set the hour in day field of the date. Does not affect any other field.
	 * @param hours
	 */
	public void setHours(int hours) {
		Calendar cal = getCalendar();
		cal.set(Calendar.HOUR_OF_DAY, hours);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
	}
	/**
	 * Set the minutes in the date. Does not affect any other field.
	 * @param minutes
	 */
	public void setMinutes(int minutes) {
		Calendar cal = getCalendar();
		cal.set(Calendar.MINUTE, minutes);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
	}
	/**
	 * Set the seconds. Will set the miilieseconds to zero.
	 * @param seconds
	 */
	public void setSeconds(int seconds) {
		Calendar cal = getCalendar();
		cal.set(Calendar.SECOND, seconds);
		cal.set(Calendar.MILLISECOND, 0);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
	}
	
	
	/**
	 * Sets the date by a parseble String. If the String fails to be parsed the result will be 
	 * <code>false</code> and the date will not be changed. The parsing will be delegated to the JaretDateFormatter. 
	 * @param text string to be parsed as date
	 * @return true if parsing is successful false otherwise
	 */
	public boolean setDateByText(String text) {
		Date date = _jaretDateFormatter.parseTextualDate(text);
		if (date == null) {
			return false;
		} else {
			_date = date;
			firePropertyChange();
			return true;
		}
	}

	/**
	 * Supply a configured (set to the current time) Calendar for internal use.
	 * @return Calendar set to the current date
	 */
	protected Calendar getCalendar() {
		if (_calendar == null) {
			_calendar = Calendar.getInstance(_jaretDateFormatter.getLocale());
		}
		_calendar.setTime(_date);
		return _calendar;
	}
	/**
	 * Calculate the Milliseconds from 0:00 ignoring the millisecond field
	 * @return Milliseconds from 0:00 ignoring millisecond field
	 */
	public long getMillisInDay() {
		Calendar cal = getCalendar();
		long h = cal.get(Calendar.HOUR_OF_DAY);
		long m = cal.get(Calendar.MINUTE);
		long s = cal.get(Calendar.SECOND);
		return h * MILLIS_PER_HOUR + m * MILLIS_PER_MINUTE + s * MILLIS_PER_SECOND; 
	}
	
	/**
	 * Returns the encapsulated java.util.Date
	 * @return the encapsulated java.util.Date
	 */
	public Date getDate(){
		return _date;
	}
	
	/**
	 * Convenient way for a quick textual output of the timestamp
	 * @return a textual representation of the date (timestamp)
	 */
	public String toDisplayString(){
		return _jaretDateFormatter.generateDisplayString(this);
	}
	/**
	 * Convenient way for quick textual output of the date (day) only
	 * @return  a textual representation of the date (day)
	 */
	public String toDisplayStringDate(){
		return _jaretDateFormatter.generateDisplayStringDate(this);
	}
	/**
	 * Convenient way for quick textual represenation (time)
	 * @param seconds if true, seconds will be included in the format hh:mm:ss
	 * @return a textual represenation of the time
	 */
	public String toDisplayStringTime(boolean seconds) {
		return _jaretDateFormatter.toDisplayStringTime(this, seconds);
	}
	/**
	 * 
	 * @return toDisplayStringTime(false)
	 */
	public String toDisplayStringTime() {
		return toDisplayStringTime(false);
	}
	
	
	/**
	 * The standard toString behaviour is the value of the date in milliseconds
	 * @return milliseconds as String
	 */
	public String toString() {
		return Long.toString(_date.getTime());
	}
	/**
	 * Sets the date to the current system time
	 */
	public void setToNow() {
		_date.setTime(System.currentTimeMillis());
		firePropertyChange();
	}
	
	
	/**
	 * Substracts the the given date from this date and returns the difference in minutes
	 * @param date the Date to be substracted  
	 * @return difference in minutes
	 */
	public double diffMinutes(JaretDate date) {
		return ((double)diffMilliSeconds(date))/(1000.0 * 60.0);
	}
	/**
	 * Substracts the the given date from this date and returns the difference in seconds
	 * @param date the Date to be substracted  
	 * @return difference in seconds
	 */
	public int diffSeconds(JaretDate date) {
		return (int)((_date.getTime() - date.getDate().getTime())/1000);
	}
	/**
	 * Substracts the the given date from this date and returns the difference in milliseconds
	 * @param date the Date to be substracted  
	 * @return difference in milliseconds
	 */
	public long diffMilliSeconds(JaretDate date) {
		return _date.getTime() - date.getDate().getTime();
	}

	
	public JaretDate advanceMonths(int months) {
		Calendar cal = getCalendar();
		cal.add(Calendar.MONTH, months);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
		return this;
	}
	public JaretDate advanceDays(int days) {
		Calendar cal = getCalendar();
		cal.add(Calendar.DAY_OF_YEAR, days);
		_date.setTime(cal.getTimeInMillis());
		firePropertyChange();
		return this;
	}
	/**
	 * Adds a number of days to the date
	 * @param days number of days to add
	 * @return the date for cascading calls
	 */
	public JaretDate advanceDays(double days) {
		_date.setTime(_date.getTime()+((long)(days * 24.0 * 60.0 * 60.0 * 1000.0)));
		firePropertyChange();
		return this;
	}

	public JaretDate backDays(double days) {
		_date.setTime(_date.getTime()-((long)(days * 24.0 * 60.0 * 60.0 * 1000.0)));
		firePropertyChange();
		return this;
	}
	
	
	public JaretDate advanceHours(double hours) {
		_date.setTime(_date.getTime()+((long)(hours * 60.0 * 60.0 * 1000.0)));
		firePropertyChange();
		return this;
	}
	
	public JaretDate backHours(double hours) {
		_date.setTime(_date.getTime()-((long)(hours * 60.0 * 60.0 * 1000.0)));
		firePropertyChange();
		return this;
	}
	
	public JaretDate advanceMinutes(double minutes) {
		_date.setTime(_date.getTime()+((long)(minutes * 60.0 * 1000.0)));
		firePropertyChange();
		return this;
	}

	public JaretDate backMinutes(double minutes) {
		_date.setTime(_date.getTime()-((long)(minutes * 60.0 * 1000.0)));
		firePropertyChange();
		return this;
	}
	
	public JaretDate advanceSeconds(double seconds) {
		_date.setTime(_date.getTime()+((long)(seconds * 1000.0)));
		firePropertyChange();
		return this;
	}

	public JaretDate backSeconds(double seconds) {
		_date.setTime(_date.getTime()-((long)(seconds * 1000.0)));
		firePropertyChange();
		return this;
	}

	
	/**
	 * Two JaretDates are equal if the encapsulated Date is equal
	 * @return true if equal
	 */
	public boolean equals(Object o) {
		return _date.equals(((JaretDate)o).getDate());
	}
	/**
	 * Hashcode is the Hash of the encapsulted Date
	 */
	public int hashCode() {
		return _date.hashCode();
	}
	/**
	 * Compare this to another JaretDate
	 * @param date JaretDate to compare to
	 * @return 0 for an exact match, >0 for this>date, <0 for this<date
	 */
	public int compareTo(JaretDate date) {
		return _date.compareTo(date._date);
	}
	/**
	 * Compares only the time of day of the date with the given day
	 * @param date the date to compare to
	 * @return 0 for an exact match, >0 for this>date, <0 for this<date
	 */
	 public long compareTimeTo(JaretDate date) {
	 	return getMillisInDay() - date.getMillisInDay();
	 }
	
	 /**
	  * Compare the date (day) of this and the given date
	  * @param date
	  * @return 0 for exact match, >0 for this>date, <0 for this<date 
	  */
	 public int compareDateTo(JaretDate date) {
	 	int result;
	 	result = getYear() - date.getYear();
	 	if (result == 0) {
	 		result = getDayOfYear() - date.getDayOfYear();
	 	}
	 	return result;
	 }
	 
	/**
	 * Produce a new JaretDate with the time of this date
	 * @return a new instance of JaretDate
	 */
	public JaretDate copy() {
		return new JaretDate(this);
	}


	
	
	public int getMonth() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.MONTH);
	}
	public String getMonthString() {
		return _jaretDateFormatter.getMonthString(getMonth());
	}
	public String getShortMonthString() {
		return _jaretDateFormatter.getShortMonthString(getMonth());
	}

	
	public int getWeekOfYear() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.WEEK_OF_YEAR);
	}
	
	/**
	 * Retrieve localized name of the current day of the week
	 * @return long name of the day of week
	 */
	public String getDayOfWeekString() {
		return _jaretDateFormatter.getDayOfWeekString(getDayOfWeek());
	}
	public String getShortDayOfWeekString() {
		return _jaretDateFormatter.getShortDayOfWeekString(getDayOfWeek());
	}
	public int getDayOfWeek() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.DAY_OF_WEEK);
	}
	public int getDay() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.DAY_OF_MONTH);
	}
	public int getHours() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.HOUR_OF_DAY);
	}
	public int getMinutes() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.MINUTE);
	}
	public int getSeconds() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.SECOND);
	}

	public int getDayOfYear() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.DAY_OF_YEAR);
	}
	/**
	 * Retrieve the year of the date
	 * @return year of the date
	 */
	public int getYear() {
		Calendar cal = getCalendar();
		return cal.get(Calendar.YEAR);
	}

	private void firePropertyChange() {
		firePropertyChange("date", null, null);
	}
	
}
