/*
 *  File: HolidayEnumeratorFactory.java 
 *  Copyright (c) 2004-2005  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.util.date.holidayenumerator;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * Factory for supplying a configured HolidayEnumerator. HolidayEnumerators are charcterized
 * by a locale and an additional RegionID. The latter is used to differentiate regions in a country and
 * should be used more finegrained than the Locale's variant. 
 * <p>
 * This factory searches in the package de.jaret.util.date.holidayenumertor for classes named "HolidayEnumerator_"+
 * locale string represenation (i.e. "de_DE"). They should have a no argument constructor or a constructor
 * taking the regionId as an argument.
 * </p>
 * @author Peter Kliem
 * @version $Id: HolidayEnumeratorFactory.java 125 2005-06-28 10:40:03Z kliem $
 */
public class HolidayEnumeratorFactory {
	// map for storing already instantiated HolidayEnumerators
	static Map<String, HolidayEnumerator> __enumerators = new HashMap<String, HolidayEnumerator>();
	/**
	 * Supply a configured HolidayEnumerator for a locale and a RegionID. The RegionID is an addition to the locale
	 * concept, allowing differentiation beetween areas in a country.
	 * @param locale Locale for the HolidayEnumerator
	 * @param regionId optional RegionID. (may be <code>null</code>).
	 * @return configured HolidayEnumerator or <code>null</code> if no 
	 * HolidayEnumerator is available for the specified locale.
	 */
	public static HolidayEnumerator getHolidayEnumeratorInstance(Locale locale, String regionId) {
		// try to find exact match
		HolidayEnumerator he = __enumerators.get(getHolidayEnumeratorId(locale, regionId));
		if (he != null) {
			return he;
		}
		// not found: try to instantiate a HolidayEnumerator
		he = instantiateHolidayEnumerator(locale, regionId);
		if (he != null) {
			__enumerators.put(getHolidayEnumeratorId(locale, regionId), he);
		}
		return he; // result may be null
	}
	/**
	 * Construct an identifying String for locale and regionId
	 * @param locale 
	 * @param regionId 
	 * @return concatenation of locae identifying string an regionId
	 */
	private static String getHolidayEnumeratorId(Locale locale, String regionId) {
		return locale.toString()+"_"+(regionId != null? regionId:"");
	}
	/**
	 * Tries to instantiate a suitable HolidayEnumeraotor for the given parameters. May ignore the regionId.
	 * @param locale
	 * @param regionId
	 * @return a HolidayEnumerator matching the parameters or null indicating no HolidayEnumerator could
	 * be instantiated.
	 */
	private static HolidayEnumerator instantiateHolidayEnumerator(Locale locale, String regionId) {
		// classname
		String classname = "de.jaret.util.date.holidayenumerator.HolidayEnumerator_"+locale.toString();
		Class clazz = null;
		try {
			clazz = Class.forName(classname);
		} catch (Exception e) {
			return null; // class not found
		}
		Constructor constructor;
		// try constructing with regionID
		try {
			constructor = clazz.getConstructor(new Class[] {String.class});
			HolidayEnumerator he = (HolidayEnumerator)constructor.newInstance(new Object[] {regionId});
			return he;
		} catch (Exception e) {
			// ignore ... next try without parameters
		}
		// if not succesful up to here try constructing without reginId
		try {
			constructor = clazz.getConstructor(new Class[] {});
			HolidayEnumerator he = (HolidayEnumerator)constructor.newInstance(new Object[] {});
			return he;
		} catch (Exception e) {
			// not succesful -> will return null
		}
		return null;
	}
	
}
